import { applyMixins, BaseNative, GestureHandlerStateEvent, GestureHandlerTouchEvent, GestureState, HandlerType, install as installBase, Manager as ManagerBase, nativeProperty, ViewDisposeEvent, ViewInitEvent, } from './gesturehandler.common';
import { observe as gestureObserve } from './gestures_override';
import { layout } from '@nativescript/core/utils/utils';
import { android as androidApp } from '@nativescript/core/application';
import { Page } from '@nativescript/core/ui/page';
import { GestureTypes } from '@nativescript/core/ui/gestures';
export { GestureState, GestureHandlerStateEvent, GestureHandlerTouchEvent, HandlerType, ViewInitEvent, ViewDisposeEvent };
let PageLayout;
class PageGestureExtended extends Page {
    initNativeView() {
        this.nativeView.initialize();
    }
    disposeNativeView() {
        this.nativeView.tearDown();
    }
    get registry() {
        return this.nativeView && this.nativeView.registry();
    }
}
let installed = false;
export function install(overrideNGestures = false) {
    if (installed) {
        return;
    }
    installed = true;
    installBase();
    const NSPage = require('@nativescript/core/ui/page').Page;
    NSPage.prototype.createNativeView = function () {
        if (!PageLayout) {
            PageLayout = com.nativescript.gesturehandler.PageLayout;
        }
        const layout = new PageLayout(this._context);
        this.gestureRegistry = layout.registry;
        return layout;
    };
    applyMixins(NSPage, [PageGestureExtended]);
    if (overrideNGestures === true) {
        const NSView = require('@nativescript/core/ui/core/view').View;
        NSView.prototype._observe = function (type, callback, thisArg) {
            if (!this._gestureObservers[type]) {
                this._gestureObservers[type] = [];
            }
            this._gestureObservers[type].push(gestureObserve(this, type, callback, thisArg));
            if (type & GestureTypes.touch && this.isLoaded && !this.touchListenerIsSet) {
                this.setOnTouchListener();
            }
        };
        NSView.prototype.setOnTouchListener = function () {
            if (!this.nativeViewProtected || !this.getGestureObservers(GestureTypes.touch)) {
                return;
            }
            this.touchListener =
                this.touchListener ||
                    new android.view.View.OnTouchListener({
                        onTouch: (view, event) => {
                            this.handleGestureTouch(event);
                            let nativeView = this.nativeViewProtected;
                            if (!nativeView || !nativeView.onTouchEvent) {
                                return false;
                            }
                            return nativeView.onTouchEvent(event);
                        },
                    });
            this.nativeViewProtected.setOnTouchListener(this.touchListener);
            this.touchListenerIsSet = true;
            if (this.nativeViewProtected.setClickable) {
                this.nativeViewProtected.setClickable(this.isUserInteractionEnabled);
            }
        };
    }
}
const KEY_HIT_SLOP_LEFT = 'left';
const KEY_HIT_SLOP_TOP = 'top';
const KEY_HIT_SLOP_RIGHT = 'right';
const KEY_HIT_SLOP_BOTTOM = 'bottom';
const KEY_HIT_SLOP_VERTICAL = 'vertical';
const KEY_HIT_SLOP_HORIZONTAL = 'horizontal';
const KEY_HIT_SLOP_WIDTH = 'width';
const KEY_HIT_SLOP_HEIGHT = 'height';
export class Handler extends BaseNative {
    constructor() {
        super(...arguments);
        this.tag = 0;
    }
    getExtraData(handler) {
        const numberOfPointers = handler.getNumberOfPointers();
        const positions = [];
        for (let index = 0; index < numberOfPointers; index++) {
            positions.push(handler.getXAtIndex(index));
            positions.push(handler.getYAtIndex(index));
        }
        return {
            positions,
            numberOfPointers,
        };
    }
    initNativeView(native, options) {
        super.initNativeView(native, options);
        this.native.setTag(this.tag);
        this.touchListener = new com.swmansion.gesturehandler.OnTouchEventListener({
            onTouchEvent: this.onTouchEvent.bind(this),
            onStateChange: this.onStateChange.bind(this),
        });
        native.setOnTouchEventListener(this.touchListener);
        this.manager.get().configureInteractions(this, options);
    }
    disposeNativeView() {
        this.native.setInteractionController(null);
        this.native.setOnTouchEventListener(null);
        this.touchListener = null;
        super.disposeNativeView();
    }
    onTouchEvent(handler, event) {
        const view = handler.getView();
        this.notify({
            eventName: GestureHandlerTouchEvent,
            object: this,
            data: {
                state: handler.getState(),
                android: view,
                extraData: this.getExtraData(handler),
                view: view.nsView ? view.nsView.get() : null,
            },
        });
    }
    onStateChange(handler, state, prevState) {
        const view = handler.getView();
        this.notify({
            eventName: GestureHandlerStateEvent,
            object: this,
            data: {
                state,
                prevState,
                android: view,
                extraData: this.getExtraData(handler),
                view: view.nsView ? view.nsView.get() : null,
            },
        });
    }
    setTag(tag) {
        this.tag = tag;
        if (this.native) {
            this.native.setTag(tag);
        }
    }
    getTag() {
        return this.tag;
    }
    getView() {
        return this.getNative().getView();
    }
    cancel() {
        return this.getNative().cancel();
    }
    attachToView(view) {
        this.manager.get().attachGestureHandler(this, view);
    }
    detachFromView(view) {
        this.manager.get().detachGestureHandler(this, view);
    }
}
__decorate([
    nativeProperty({
        nativeSetterName: 'setHitSlop',
        nativegetterName: 'getHitSlop',
        converter: {
            toNative(value) {
                const HIT_SLOP_NONE = com.swmansion.gesturehandler.GestureHandler.HIT_SLOP_NONE;
                if (typeof value === 'number') {
                    const hitSlop = layout.toDevicePixels(value);
                    return [hitSlop, hitSlop, hitSlop, hitSlop, HIT_SLOP_NONE, HIT_SLOP_NONE];
                }
                else {
                    let left = HIT_SLOP_NONE, top = HIT_SLOP_NONE, right = HIT_SLOP_NONE, bottom = HIT_SLOP_NONE;
                    let width = HIT_SLOP_NONE, height = HIT_SLOP_NONE;
                    if (value.hasOwnProperty(KEY_HIT_SLOP_HORIZONTAL)) {
                        const horizontalPad = layout.toDevicePixels(value[KEY_HIT_SLOP_HORIZONTAL]);
                        left = right = horizontalPad;
                    }
                    if (value.hasOwnProperty(KEY_HIT_SLOP_VERTICAL)) {
                        const verticalPad = layout.toDevicePixels(value[KEY_HIT_SLOP_VERTICAL]);
                        top = bottom = verticalPad;
                    }
                    if (value.hasOwnProperty(KEY_HIT_SLOP_LEFT)) {
                        left = layout.toDevicePixels(value[KEY_HIT_SLOP_LEFT]);
                    }
                    if (value.hasOwnProperty(KEY_HIT_SLOP_TOP)) {
                        top = layout.toDevicePixels(value[KEY_HIT_SLOP_TOP]);
                    }
                    if (value.hasKey(KEY_HIT_SLOP_RIGHT)) {
                        right = layout.toDevicePixels(value[KEY_HIT_SLOP_RIGHT]);
                    }
                    if (value.hasOwnProperty(KEY_HIT_SLOP_BOTTOM)) {
                        bottom = layout.toDevicePixels(value[KEY_HIT_SLOP_BOTTOM]);
                    }
                    if (value.hasOwnProperty(KEY_HIT_SLOP_WIDTH)) {
                        width = layout.toDevicePixels(value[KEY_HIT_SLOP_WIDTH]);
                    }
                    if (value.hasOwnProperty(KEY_HIT_SLOP_HEIGHT)) {
                        height = layout.toDevicePixels(value[KEY_HIT_SLOP_HEIGHT]);
                    }
                    return [left, top, right, bottom, width, height];
                }
            },
        },
    })
], Handler.prototype, "hitSlop", void 0);
__decorate([
    nativeProperty
], Handler.prototype, "enabled", void 0);
__decorate([
    nativeProperty
], Handler.prototype, "shouldCancelWhenOutside", void 0);
export class TapGestureHandler extends Handler {
    createNative(options) {
        return new com.swmansion.gesturehandler.TapGestureHandler();
    }
    getExtraData(handler) {
        return Object.assign(super.getExtraData(handler), {
            x: layout.toDeviceIndependentPixels(handler.getLastRelativePositionX()),
            y: layout.toDeviceIndependentPixels(handler.getLastRelativePositionY()),
            absoluteX: layout.toDeviceIndependentPixels(handler.getLastAbsolutePositionX()),
            absoluteY: layout.toDeviceIndependentPixels(handler.getLastAbsolutePositionY()),
        });
    }
}
__decorate([
    nativeProperty
], TapGestureHandler.prototype, "numberOfTaps", void 0);
__decorate([
    nativeProperty
], TapGestureHandler.prototype, "maxDurationMs", void 0);
__decorate([
    nativeProperty
], TapGestureHandler.prototype, "maxDelayMs", void 0);
__decorate([
    nativeProperty({ nativeSetterName: 'setMaxDx' })
], TapGestureHandler.prototype, "maxDeltaX", void 0);
__decorate([
    nativeProperty({ nativeSetterName: 'setMaxDy' })
], TapGestureHandler.prototype, "maxDeltaY", void 0);
__decorate([
    nativeProperty({ converter: { fromNative: layout.toDevicePixels } })
], TapGestureHandler.prototype, "maxDist", void 0);
__decorate([
    nativeProperty({ nativeSetterName: 'setMinNumberOfPointers' })
], TapGestureHandler.prototype, "minPointers", void 0);
export class PanGestureHandler extends Handler {
    createNative(options) {
        const context = androidApp.context;
        return new com.swmansion.gesturehandler.PanGestureHandler(context);
    }
    getExtraData(handler) {
        return Object.assign(super.getExtraData(handler), {
            x: layout.toDeviceIndependentPixels(handler.getLastRelativePositionX()),
            y: layout.toDeviceIndependentPixels(handler.getLastRelativePositionY()),
            absoluteX: layout.toDeviceIndependentPixels(handler.getLastAbsolutePositionX()),
            absoluteY: layout.toDeviceIndependentPixels(handler.getLastAbsolutePositionY()),
            translationX: layout.toDeviceIndependentPixels(handler.getTranslationX()),
            translationY: layout.toDeviceIndependentPixels(handler.getTranslationY()),
            velocityX: layout.toDeviceIndependentPixels(handler.getVelocityX()),
            velocityY: layout.toDeviceIndependentPixels(handler.getVelocityY()),
        });
    }
}
__decorate([
    nativeProperty({ converter: { fromNative: layout.toDevicePixels } })
], PanGestureHandler.prototype, "minDist", void 0);
__decorate([
    nativeProperty({ converter: { fromNative: layout.toDevicePixels } })
], PanGestureHandler.prototype, "activeOffsetXStart", void 0);
__decorate([
    nativeProperty({ converter: { fromNative: layout.toDevicePixels } })
], PanGestureHandler.prototype, "activeOffsetXEnd", void 0);
__decorate([
    nativeProperty({ converter: { fromNative: layout.toDevicePixels } })
], PanGestureHandler.prototype, "failOffsetXStart", void 0);
__decorate([
    nativeProperty({ converter: { fromNative: layout.toDevicePixels } })
], PanGestureHandler.prototype, "failOffsetXEnd", void 0);
__decorate([
    nativeProperty({ converter: { fromNative: layout.toDevicePixels } })
], PanGestureHandler.prototype, "activeOffsetYStart", void 0);
__decorate([
    nativeProperty({ converter: { fromNative: layout.toDevicePixels } })
], PanGestureHandler.prototype, "activeOffsetYEnd", void 0);
__decorate([
    nativeProperty({ converter: { fromNative: layout.toDevicePixels } })
], PanGestureHandler.prototype, "failOffsetYStart", void 0);
__decorate([
    nativeProperty({ converter: { fromNative: layout.toDevicePixels } })
], PanGestureHandler.prototype, "failOffsetYEnd", void 0);
__decorate([
    nativeProperty({ converter: { fromNative: layout.toDevicePixels } })
], PanGestureHandler.prototype, "minVelocity", void 0);
__decorate([
    nativeProperty({ converter: { fromNative: layout.toDevicePixels } })
], PanGestureHandler.prototype, "minVelocityX", void 0);
__decorate([
    nativeProperty({ converter: { fromNative: layout.toDevicePixels } })
], PanGestureHandler.prototype, "minVelocityY", void 0);
__decorate([
    nativeProperty
], PanGestureHandler.prototype, "minPointers", void 0);
__decorate([
    nativeProperty
], PanGestureHandler.prototype, "maxPointers", void 0);
__decorate([
    nativeProperty({ nativeSetterName: 'setAverageTouches' })
], PanGestureHandler.prototype, "avgTouches", void 0);
__decorate([
    nativeProperty
], PanGestureHandler.prototype, "numberOfPointers", void 0);
export class PinchGestureHandler extends Handler {
    createNative(options) {
        return new com.swmansion.gesturehandler.PinchGestureHandler();
    }
    getExtraData(handler) {
        return Object.assign(super.getExtraData(handler), {
            scale: handler.getScale(),
            focalX: layout.toDeviceIndependentPixels(handler.getFocalPointX()),
            focalY: layout.toDeviceIndependentPixels(handler.getFocalPointY()),
            velocity: handler.getVelocity(),
        });
    }
}
__decorate([
    nativeProperty({ converter: { fromNative: layout.toDevicePixels } })
], PinchGestureHandler.prototype, "minSpan", void 0);
function directionToString(direction) {
    switch (direction) {
        case com.swmansion.gesturehandler.GestureHandler.DIRECTION_RIGHT:
            return 'right';
        case com.swmansion.gesturehandler.GestureHandler.DIRECTION_UP:
            return 'up';
        case com.swmansion.gesturehandler.GestureHandler.DIRECTION_DOWN:
            return 'down';
        default:
            return 'left';
    }
}
function directionFromString(direction) {
    switch (direction) {
        case 'right':
            return com.swmansion.gesturehandler.GestureHandler.DIRECTION_RIGHT;
        case 'up':
            return com.swmansion.gesturehandler.GestureHandler.DIRECTION_UP;
        case 'down':
            return com.swmansion.gesturehandler.GestureHandler.DIRECTION_DOWN;
        default:
            return com.swmansion.gesturehandler.GestureHandler.DIRECTION_LEFT;
    }
}
export class FlingGestureHandler extends Handler {
    createNative(options) {
        return new com.swmansion.gesturehandler.FlingGestureHandler();
    }
    getExtraData(handler) {
        return Object.assign(super.getExtraData(handler), {
            direction: directionToString(handler.getRecognizedDirection()),
        });
    }
}
__decorate([
    nativeProperty
], FlingGestureHandler.prototype, "numberOfPointers", void 0);
__decorate([
    nativeProperty({ converter: { fromNative: directionToString, toNative: directionFromString } })
], FlingGestureHandler.prototype, "direction", void 0);
export class LongPressGestureHandler extends Handler {
    createNative(options) {
        const context = androidApp.context;
        return new com.swmansion.gesturehandler.LongPressGestureHandler(context);
    }
}
__decorate([
    nativeProperty
], LongPressGestureHandler.prototype, "minDurationMs", void 0);
__decorate([
    nativeProperty({ converter: { fromNative: layout.toDevicePixels } })
], LongPressGestureHandler.prototype, "maxDist", void 0);
export class RotationGestureHandler extends Handler {
    createNative(options) {
        return new com.swmansion.gesturehandler.RotationGestureHandler();
    }
    getExtraData(handler) {
        return Object.assign(super.getExtraData(handler), {
            rotation: handler.getRotation(),
            anchorX: layout.toDeviceIndependentPixels(handler.getAnchorX()),
            anchorY: layout.toDeviceIndependentPixels(handler.getAnchorY()),
            velocity: handler.getVelocity(),
        });
    }
}
export class NativeViewGestureHandler extends Handler {
    createNative(options) {
        return new com.swmansion.gesturehandler.NativeViewGestureHandler();
    }
    getExtraData(handler) {
        return Object.assign(super.getExtraData(handler), {
            pointerInside: handler.isWithinBounds(),
        });
    }
}
__decorate([
    nativeProperty
], NativeViewGestureHandler.prototype, "shouldActivateOnStart", void 0);
__decorate([
    nativeProperty
], NativeViewGestureHandler.prototype, "disallowInterruption", void 0);
export class Manager extends ManagerBase {
    constructor() {
        super(...arguments);
        this.viewListeners = new Map();
    }
    configureInteractions(handler, options = {}) {
        this.interactionManager.configureInteractions(handler.getNative(), options.waitFor, options.simultaneousHandlers);
    }
    get interactionManager() {
        if (!this._interactionManager) {
            this._interactionManager = new com.nativescript.gesturehandler.GestureHandlerInteractionController();
        }
        return this._interactionManager;
    }
    static getInstance() {
        if (!Manager.sManager) {
            Manager.sManager = new Manager();
        }
        return Manager.sManager;
    }
    createGestureHandler(handlerName, handlerTag, config) {
        let handler = null;
        switch (handlerName) {
            case 'tap':
                handler = new TapGestureHandler(config);
                break;
            case 'pan':
                handler = new PanGestureHandler(config);
                break;
            case 'nativeView':
                handler = new NativeViewGestureHandler(config);
                break;
            case 'pinch':
                handler = new PinchGestureHandler(config);
                break;
            case 'fling':
                handler = new FlingGestureHandler(config);
                break;
            case 'rotation':
                handler = new RotationGestureHandler(config);
                break;
            case 'longPress':
                handler = new LongPressGestureHandler(config);
                break;
        }
        if (handler) {
            handler.manager = new WeakRef(this);
            handler.setTag(handlerTag);
        }
        return handler;
    }
    attachGestureHandlerToView(handler, view) {
        const nHandler = handler.getNative();
        if (nHandler) {
            const page = view.page;
            if (page) {
                const registry = page.registry;
                if (registry) {
                    registry.registerHandler(nHandler);
                    registry.attachHandlerToView(nHandler.getTag(), view.nativeView);
                }
            }
            else {
                throw new Error('a page is needed to attach a gesture');
            }
        }
    }
    detachGestureHandlerFromView(handler, view) {
        const nHandler = handler.getNative();
        if (nHandler) {
            const registry = view.page && view.page.registry;
            if (registry) {
                registry.dropHandler(handler.getNative());
            }
        }
    }
    attachGestureHandler(handler, view) {
        if (view.nativeView) {
            this.attachGestureHandlerToView(handler, view);
        }
        const onInit = () => this.attachGestureHandlerToView(handler, view);
        const onDispose = () => this.detachGestureHandlerFromView(handler, view);
        view.on(ViewInitEvent, onInit, this);
        view.on(ViewDisposeEvent, onDispose, this);
        let viewListeners = this.viewListeners.get(view);
        if (!viewListeners) {
            viewListeners = new Map();
            this.viewListeners.set(view, viewListeners);
        }
        viewListeners.set(handler.getTag(), {
            init: onInit,
            dispose: onDispose,
        });
    }
    detachGestureHandler(handler, view) {
        if (view) {
            const viewListeners = this.viewListeners.get(view);
            if (viewListeners) {
                const listeners = viewListeners.get(handler.getTag());
                if (listeners) {
                    view.off(ViewInitEvent, listeners.init, this);
                    view.off(ViewDisposeEvent, () => listeners.dispose, this);
                    viewListeners.delete(handler.getTag());
                    if (viewListeners.size === 0) {
                        this.viewListeners.delete(view);
                    }
                }
            }
        }
        this.detachGestureHandlerFromView(handler, view);
    }
}
//# sourceMappingURL=gesturehandler.android.mjs.map